#include "state_area_report_generator.h"

 
void Xport_report_suite::state_area_report_generator::create_header(document& doc) const
{
  *doc.title() << "State Area Report";

  element hdr(Xport::div, "id_header");
  hdr << (element(h1) << "State Area Report");
  
  element lnk(a);
  lnk << "Wikipedia";
  lnk.attribute(href, "http://en.wikipedia.org/wiki/Wiki");
  hdr << (element(p) << "Statistics and images courtesy of " << lnk);

  *doc.body() << hdr;
}

void Xport_report_suite::state_area_report_generator::create_report_body(document& doc) const
{
  std::vector<state_info> siv;
  us_state_report_generator::get_state_data(siv, sf_area, descending);
  std::vector<state_info>::const_iterator it = siv.begin();

  element tbl(table);
  create_table_header(tbl);

  for (; it != siv.end(); ++it) {
    element row(tr);
    row << (element(td, "", "area_rank") << numeric_format(it->area_rank()));
    row << (element(td, "", "state") << it->name());
    row << (element(td, "", "abbr") << it->abbr());
    row << (element(td, "", "area") << numeric_format(it->area(), 2));

    tbl << row;
  }

  *doc.body() << tbl;
}

void Xport_report_suite::state_area_report_generator::create_table_header(element& tbl) const
{
  element row(tr, "", "titles");
  row << (element(th) << "Area Rank");
  row << (element(th) << "State");
  row << (element(th) << "Abbr");
  row << (element(th, "", "area") << "Area");
  tbl << row;
}


void Xport_report_suite::state_area_report_generator::create_footer(document& doc) const
{
  markup::iterator it = doc.body()->insert(element(Xport::div, "id_footer"));
  it = it->insert(element(p));
  *it << "Report and stylesheet created by ";
  it = it->insert(element(a));
  *it << "Xport";
  it->attribute(href, "http://www.xportpro.com");
}

void Xport_report_suite::state_area_report_generator::create_stylesheet(document& doc, stylesheet& ss) const
{
  stylesheet::iterator it = ss.insert(stylesheet_rule("#id_header"));
  *it << declaration(position, "relative")
      << declaration(css::height, "100px")
      << declaration(css::width, "70%")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(background_color, "#b4af91")
      << declaration(margin_bottom, "50px")
      << declaration(border_bottom, "2px solid #32331d")
      << declaration(border_right, "2px solid #32331d")
      << declaration(border_left, "40px ridge #b4af91")
      << declaration(border_top, "2px solid #32331d");

  it = ss.insert(stylesheet_rule("#id_header h1"));
  *it << declaration(text_align, "center")
      << declaration(padding_top, "20px")
      << declaration(css::margin, "0px");

  it = ss.insert(stylesheet_rule("#id_header p"));
  *it << declaration(position, "absolute")
      << declaration(right, "10px")
      << declaration(bottom, "5px")
      << declaration(css::margin, "0px");

  it = ss.insert(stylesheet_rule("table"));
  *it << declaration(css::width, "70%")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(border_collapse, "collapse")
      << declaration(border_bottom, "2px solid #32331d")
      << declaration(border_right, "2px solid #32331d")
      << declaration(border_left, "40px ridge #b4af91")
      << declaration(border_top, "2px solid #32331d");

  it = ss.insert(stylesheet_rule("th"));
  *it << declaration(border_bottom, "2px solid #32331df")
      << declaration(line_height, "200%")
      << declaration(background_color, "#40411e")
      << declaration(css::color, "white")
      << declaration(text_align, "left")
      << declaration(padding_left, "10px");

  it = ss.insert(stylesheet_rule("td"));
  *it << declaration(padding_left, "15px")
      << declaration(line_height, "150%")
      << declaration(border_bottom, "1px solid #40411e")
      << declaration(background_color, "#b4af91");

  it = ss.insert(stylesheet_rule("td.area_rank"));
  *it << declaration(background_color, "#c03000")
      << declaration(css::width, "70px")
      << declaration(border_right, "1px solid #40411e")
      << declaration(css::color, "#32331d")
      << declaration(font_weight, "bold")
      << declaration(text_align, "center");

  it = ss.insert(stylesheet_rule("td.state"));
  *it << declaration(css::width, "20%");

  it = ss.insert(stylesheet_rule(".area"));
  *it << declaration(text_align, "right")
      << declaration(padding_right, "50px")
      << declaration(font_weight, "bold");

  it = ss.insert(stylesheet_rule("th.area"));
  *it << declaration(padding_right, "80px");

  it = ss.insert(stylesheet_rule("#id_footer"));
  *it << declaration(css::width, "70%")
      << declaration(css::height, "40px")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(margin_top, "70px")
      << declaration(border_top, "2px solid #32331d")
      << declaration(border_bottom, "2px solid #32331d")
      << declaration(border_right, "2px solid #32331d")
      << declaration(border_left, "40px ridge #b4af91")
      << declaration(background_color, "#b4af91");

  it = ss.insert(stylesheet_rule("#id_footer p"));
  *it << declaration(text_align, "center")
      << declaration(padding_top, "10px")
      << declaration(margin, "0px");

  element css_link(Xport::link);
  css_link.attribute(rel, "stylesheet");
  css_link.attribute(type, "text/css");
  std::ostringstream file_name;
  file_name << report_name() << ".css";
  css_link.attribute(href, file_name.str());
  *doc.head() << css_link;
}
