#pragma once

#include <string>
#include <vector>
#include <fstream>
#include <sstream>
#include <algorithm>

namespace Xport_report_suite 
{
  class state_info;
  void load_state_data(const std::string& file_path, std::vector<state_info>& siv);
  void extract_data(const std::string& line, std::vector<state_info>& siv);

  enum us_region { northeast = 1, midwest, south, west };
  enum us_division { new_england = 1, middle_atlantic, east_north_central, west_north_central, south_atlantic, 
              east_south_central, west_south_central, mountain, pacific };
}

class Xport_report_suite::state_info
{
public:
  state_info( const std::string& st_name,
        const std::string& st_abbr,
        const std::string& st_capital, 
        const std::string& st_largest_city,
        const std::string& st_nickname,
        const std::string& st_slogan,
        const std::string& st_tree,
        const std::string& st_flower,
        int st_population,
        int st_population_rank,
        long st_area,
        int st_area_rank,
        double st_density,
        int st_density_rank,
        double st_percent_water,
        const std::string st_admit_date,
        const std::string st_web_site,
        int st_region,
        int st_division)
        : state_name(st_name),
          state_abbr(st_abbr),
          state_capital(st_capital),
          state_largest_city(st_largest_city),
          state_nickname(st_nickname),
          state_slogan(st_slogan),
          state_tree(st_tree),
          state_flower(st_flower),
          state_population(st_population),
          state_population_rank(st_population_rank),
          state_area(st_area),
          state_area_rank(st_area_rank),
          state_density(st_density),
          state_density_rank(st_density_rank),
          state_percent_water(st_percent_water),
          state_admit_date(st_admit_date),
          state_web_site(st_web_site), 
          state_region(static_cast<us_region>(st_region)),
          state_division(static_cast<us_division>(st_division)) {}
 
  std::string name() const { return state_name; }
  std::string abbr() const { return state_abbr; }
  std::string capital() const { return state_capital; }
  std::string largest_city() const { return state_largest_city; }
  std::string nickname() const { return state_nickname; }
  std::string slogan() const { return state_slogan; }
  std::string tree() const { return state_tree; }
  std::string flower() const { return state_flower; }
  int population() const { return state_population; }
  int population_rank() const { return state_population_rank; }
  long area() const { return state_area; }
  int area_rank() const { return state_area_rank; }
  double density() const { return state_density; }
  int density_rank() const { return state_density_rank; }
  double percent_water() const { return state_percent_water; }
  std::string admit_date() const { return state_admit_date; }
  std::string web_site() const { return state_web_site; }
  us_region region() const { return state_region; }
  us_division division() const { return state_division; }


private:
  std::string state_name;
  std::string state_abbr;
  std::string state_capital;
  std::string state_largest_city;
  std::string state_nickname;
  std::string state_slogan;
  std::string state_tree;
  std::string state_flower;
  int state_population;
  int state_population_rank;
  long state_area;
  int state_area_rank;
  double state_density;
  int state_density_rank;
  double state_percent_water;
  std::string state_admit_date;
  std::string state_web_site;
  us_region state_region;
  us_division state_division;
};

