#include "state_density_report_generator.h"

 
void Xport_report_suite::state_density_report_generator::create_header(document& doc) const
{
  *doc.title() << "State Density Report";

  element header(Xport::div, "id_header");
  header << (element(h1) << "State Density Report");
  element lnk(a);
  lnk << "Wikipedia";
  lnk.attribute(href, "http://en.wikipedia.org/wiki/Wiki");
  header << (element(p) << "Statistics and images courtesy of " << lnk);
  *doc.body() << header;
}

void Xport_report_suite::state_density_report_generator::create_report_body(document& doc) const
{
  // retrieve data
  std::vector<state_info> siv;
  us_state_report_generator::get_state_data(siv, sf_density, descending);
  std::vector<state_info>::const_iterator it = siv.begin();

  element tbl(table);
  create_table_header(tbl);

  // add table data
  for (; it != siv.end(); ++it) {
    element row(tr, "", (it - siv.begin()) % 2 ? "even" : "odd");
    row << (element(td, "", "state_name") << it->name());
    row << (element(td, "", "state_abbr") << it->abbr());
    row << (element(td, "", "density") << numeric_format(it->density(), 2));
    row << (element(td, "", "density_rank") << numeric_format(it->density_rank()));
    tbl << row;
  }

  *doc.body() << tbl;
}

void Xport_report_suite::state_density_report_generator::create_table_header(element& tbl) const
{
  element tbl_hdr(tr);
  tbl_hdr << (element(th, "", "state_name") << "State");
  tbl_hdr << (element(th, "", "state_abbr") << "Abbr");
  tbl_hdr << (element(th, "", "density") << "Density");
  tbl_hdr << (element(th, "", "density_rank") << "Density Rank");

  tbl << tbl_hdr;
}


void Xport_report_suite::state_density_report_generator::create_footer(document& doc) const
{
  element footer(Xport::div, "id_footer");
  element lnk(a);
  lnk << "Xport";
  lnk.attribute(href, "http://www.xportpro.com");
  footer << (element(p) << "Report and stylesheet created by " << lnk);
  *doc.body() << footer;
}

void Xport_report_suite::state_density_report_generator::create_stylesheet(document& doc, stylesheet& ss) const
{
  stylesheet::iterator it = ss.insert(stylesheet_rule("body"));
  *it << declaration(background_color, "#ffffcc");

  it = ss.insert(stylesheet_rule("#id_header"));
  *it << declaration(css::width, "70%")
      << declaration(css::height, "150px")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(background_color, "#999933")
      << declaration(css::border, "5px ridge #996633")
      << declaration(position, "relative")
      << declaration(margin_bottom, "50px");

  it = ss.insert(stylesheet_rule("#id_header h1"));
  *it << declaration(text_align, "center")
      << declaration(padding_top, "40px")
      << declaration(margin_top, "0px")
      << declaration(css::color, "#0000cc");

  it = ss.insert(stylesheet_rule("#id_header p"));
  *it << declaration(css::position, "absolute")
      << declaration(bottom, "10px")
      << declaration(right, "20px")
      << declaration(font_style, "italic")
      << declaration(margin_bottom, "0px");

  it = ss.insert(stylesheet_rule("table"));
  *it << declaration(css::width, "70%")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(css::border, "5px ridge #996633")
      << declaration(border_collapse, "collapse");

  it = ss.insert(stylesheet_rule("th"));
  *it << declaration(css::color, "#993333")
      << declaration(text_align, "left")
      << declaration(padding_left, "10px")
      << declaration(line_height, "150%")
      << declaration(background_color, "#669966")
      << declaration(border_bottom, "2px solid black");

  it = ss.insert(stylesheet_rule("tr"));
  *it << declaration(line_height, "150%");

  it = ss.insert(stylesheet_rule("td"));
  *it << declaration(padding_left, "10px");

  it = ss.insert(stylesheet_rule("tr.odd td"));
  *it << declaration(background_color, "#99ffcc");

  it = ss.insert(stylesheet_rule("tr.even td"));
  *it << declaration(background_color, "#cccc99");

  it = ss.insert(stylesheet_rule("td.density_rank"));
  *it << declaration(padding_left, "30px");

  it = ss.insert(stylesheet_rule("#id_footer"));
  *it << declaration(css::width, "70%")
      << declaration(css::height, "40px")
      << declaration(margin_left, "auto")
      << declaration(margin_right, "auto")
      << declaration(margin_top, "40px")
      << declaration(background_color, "#99cc99")
      << declaration(css::border, "5px ridge #996633");

  it = ss.insert(stylesheet_rule("#id_footer p"));
  *it << declaration(text_align, "center")
      << declaration(padding_top, "10px")
      << declaration(margin_top, "0px");

  // add link to stylesheet file in head element
  element css_link(Xport::link);
  css_link.attribute(rel, "stylesheet");
  css_link.attribute(type, "text/css");
  css_link.attribute(href, "state_density_report.css");
  *doc.head() << css_link;
}

