#include "us_state_report_generator.h"
#include <iomanip>

 
namespace Xport_report_suite
{
  // NOTE!  Modify these strings for your own preferences
  const std::string data_path = "c:/us_state_reports/data/states.txt";
  const std::string report_root_dir = "c:/us_state_reports";
}

Xport_report_suite::us_state_report_generator::us_state_report_generator()
{
  load_state_data(data_path, state_info_vec);
}

Xport_report_suite::us_state_report_generator::~us_state_report_generator()
{

}

void Xport_report_suite::us_state_report_generator::generate_report(document& doc, stylesheet& ss) const
{
  create_header(doc);
  create_report_body(doc);
  create_footer(doc);
  create_stylesheet(doc, ss);
}


void Xport_report_suite::us_state_report_generator::get_state_data(std::vector<state_info>& siv, state_sort_field sf, sort_order so) const
{
  siv = state_info_vec;

  // sort regional states by sort order
  std::sort(siv.begin(), siv.end(), state_comp(sf, so));
}

void Xport_report_suite::us_state_report_generator::get_divisional_state_data(int divis, std::vector<state_info>& siv, state_sort_field sf, sort_order so) const
{
  if (current_sort_field != sf_division) {
    std::sort(state_info_vec.begin(), state_info_vec.end(), state_comp(sf_division, ascending));
    current_sort_field = sf_division;
  }

  std::vector<state_info>::const_iterator it = std::find_if(state_info_vec.begin(), state_info_vec.end(), division_up(divis));

  for (; it != state_info_vec.end(); ++it) {
    if (it->division() != divis) {
      break;
    }
    siv.push_back(*it);
  }

  // sort regional states by sort order
  std::sort(siv.begin(), siv.end(), state_comp(sf, so));
  
}

void Xport_report_suite::us_state_report_generator::get_regional_state_data(int rgn, std::vector<state_info>& siv, state_sort_field sf, sort_order so) const
{
  if (current_sort_field != sf_region) {
    std::sort(state_info_vec.begin(), state_info_vec.end(), state_comp(sf_region, ascending));
    current_sort_field = sf_region;
  }

  std::vector<state_info>::const_iterator it = std::find_if(state_info_vec.begin(), state_info_vec.end(), region_up(rgn));

  for (; it != state_info_vec.end(); ++it) {
    if (it->region() != rgn) {
      break;
    }
    siv.push_back(*it);
  }

  // sort regional states by sort order
  std::sort(siv.begin(), siv.end(), state_comp(sf, so));
}


std::string Xport_report_suite::us_state_report_generator::get_division_name(int division) const
{
  switch(division) 
  {
  case new_england: return "New England"; break;
  case middle_atlantic: return "Middle Atlantic"; break;
  case east_north_central: return "East North Central"; break;
  case west_north_central: return "West North Central"; break;
  case south_atlantic: return "South Atlantic"; break;
  case east_south_central: return "East South Central"; break;
  case west_south_central: return "West South Central"; break;
  case mountain: return "Mountain"; break;
  case pacific: return "Pacific"; break;
  default: return ""; break;
  }
  return "";

}

std::string Xport_report_suite::us_state_report_generator::get_region_name(int region) const
{
  switch(region) 
  {
  case northeast: return "Northeast"; break;
  case midwest: return "Midwest"; break;
  case south: return "South"; break;
  case west: return "West"; break;
  default: return ""; break;
  }
  return "";
}


std::string Xport_report_suite::us_state_report_generator::numeric_format(double num, int dec) const
{
  std::ostringstream sNum;
  sNum << std::fixed << std::setprecision(dec) << num;
  std::string result(sNum.str());
  std::string::reverse_iterator rit = result.rbegin(), rit_end = result.rend();

  std::string result2;
  std::string::size_type dec_idx = result.find('.');
  if (dec_idx != std::string::npos) {
    rit += result.size() - dec_idx;
    result2 = result.substr(dec_idx);
    std::reverse(result2.begin(), result2.end());
  }

  int thous_idx = 1;
  for (; rit != rit_end; ++rit, ++thous_idx) {
    result2.push_back(*rit);
    if (thous_idx % 3 == 0) {
      result2.push_back(',');
    }
  }

  std::reverse(result2.begin(), result2.end());

  std::string::iterator it = result2.begin();
  if (it != result2.end() && *it == ',') {
    result2.erase(result2.begin());
  }

  return result2;
}

std::string Xport_report_suite::us_state_report_generator::numeric_format(int num) const
{
  return numeric_format(num, 0);
}

std::string Xport_report_suite::us_state_report_generator::report_path() const
{
  std::ostringstream path;
  path << report_root_dir << "/" << report_dir() << "/" << report_name();
  return path.str();
}

void Xport_report_suite::us_state_report_generator::create_directory() const
{
  #ifdef WIN32
    mkdir (report_root_dir.c_str()); // insure report root folder exists
    std::ostringstream subdir;
    subdir << report_root_dir << "/" << report_dir();
    mkdir(subdir.str().c_str());
  #else
    mkdir (report_root_dir.c_str(), 0777); // insure report root folder exists
    std::ostringstream subdir;
    subdir << report_root_dir << "/" << report_dir();
    mkdir(subdir.str().c_str(), 0777);
  #endif
}
